{% extends "admin/_base.html" %}

{% from "macros/box.html" import m_box %}

{% block content %}
  {%- call m_box(title=_("Settings")) %}
    <form method="POST" role="form">
      {{ csrf.field() }}

      {%- for k in keys %}
        <div class="form-group">
          <label for="{{ k.id }}">{{ k.label or k.id }}</label>
          {%-if k.description %}
            <p class="help-block">{{ k.description }}</p>
          {%- endif %}
          {{ k }}
        </div>
      {%- endfor %}

      <button class="btn btn-primary" type="submit" name="action" value="save">{{ _("Save") }}</button>
    </form>
  {%- endcall %}
{% endblock %}
