{% extends base_template %}

{% from "macros/box.html" import m_box_content with context %}
{% from "macros/form.html" import m_form %}

{% block content %}
  {%- block before_form %}
  {%- endblock %}

  {%- call m_box_content(view.title) %}
    {{ m_form(form, action=request.path, id="edit-form", horizontal=True) }}
  {%- endcall %}

  {%- block after_form %}
  {%- endblock %}
{% endblock %}
